@@ -10,8 +10,10 @@ Follow-On (GRACE-FO) missions
 
 Documentation is available at https://model-harmonics.readthedocs.io
 """
+import model_harmonics.spatial
 import model_harmonics.utilities
 import model_harmonics.version
+from model_harmonics.constants import constants
 from model_harmonics.gen_atmosphere_stokes import gen_atmosphere_stokes
 from model_harmonics.gen_point_pressure import gen_point_pressure
 from model_harmonics.gen_pressure_stokes import gen_pressure_stokes
