@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 reanalysis_monthly_harmonics.py
-Written by Tyler Sutterley (05/2022)
+Written by Tyler Sutterley (12/2022)
 Reads atmospheric surface pressure fields from reanalysis and calculates sets of
     spherical harmonics using a thin-layer 2D spherical geometry
 
@@ -48,9 +48,9 @@ PYTHON DEPENDENCIES:
         https://www.h5py.org/
 
 PROGRAM DEPENDENCIES:
-    plm_holmes.py: computes fully-normalized associated Legendre polynomials
+    associated_legendre.py: computes fully-normalized associated Legendre polynomials
     read_love_numbers.py: reads Load Love Numbers from Han and Wahr (1995)
-    ref_ellipsoid.py: calculate reference parameters for common ellipsoids
+    constants.py: calculate reference parameters for common ellipsoids
     gen_stokes.py: converts a spatial field into a series of spherical harmonics
     harmonics.py: spherical harmonic data class for processing GRACE/GRACE-FO
     destripe_harmonics.py: calculates the decorrelation (destriping) filter
@@ -70,6 +70,9 @@ REFERENCES:
         https://doi.org/10.1029/2000JB000024
 
 UPDATE HISTORY:
+    Updated 12/2022: single implicit import of spherical harmonic tools
+        use constants class in place of geoid-toolkit ref_ellipsoid
+    Updated 11/2022: use f-strings for formatting verbose or ascii output
     Updated 05/2022: use argparse descriptions within sphinx documentation
     Updated 04/2022: use wrapper function for reading load Love numbers
     Updated 12/2021: can use variable loglevels for verbose output
@@ -106,253 +109,255 @@ import netCDF4
 import argparse
 import datetime
 import numpy as np
-import gravity_toolkit.time
-import gravity_toolkit.harmonics
-import gravity_toolkit.utilities as utilities
-from gravity_toolkit.read_love_numbers import load_love_numbers
-from gravity_toolkit.plm_holmes import plm_holmes
-from gravity_toolkit.gen_stokes import gen_stokes
-from geoid_toolkit.ref_ellipsoid import ref_ellipsoid
+import gravity_toolkit as gravtk
+import model_harmonics as mdlhmc
 
-#-- PURPOSE: read atmospheric surface pressure fields and convert to harmonics
+# PURPOSE: read atmospheric surface pressure fields and convert to harmonics
 def reanalysis_monthly_harmonics(base_dir, MODEL, YEARS, RANGE=None,
     REDISTRIBUTE=False, LMAX=0, MMAX=None, LOVE_NUMBERS=0, REFERENCE=None,
     DATAFORM=None, MODE=0o775):
 
-    #-- directory setup
+    # directory setup
     ddir = os.path.join(base_dir,MODEL)
 
-    #-- set model specific parameters
+    # set model specific parameters
     if (MODEL == 'ERA-Interim'):
-        #-- mean file from calculate_mean_pressure.py
+        # mean file from calculate_mean_pressure.py
         input_mean_file = 'ERA-Interim-Mean-SP-{0:4d}-{1:4d}.nc'
-        #-- input land-sea mask for ocean redistribution
+        # input land-sea mask for ocean redistribution
         input_mask_file = 'ERA-Interim-Invariant-Parameters.nc'
-        #-- regular expression pattern for finding files
+        # regular expression pattern for finding files
         regex_pattern = r'ERA\-Interim\-Monthly\-SP\-({0})\.nc$'
         VARNAME = 'sp'
         LONNAME = 'longitude'
         LATNAME = 'latitude'
         TIMENAME = 'time'
         ELLIPSOID = 'WGS84'
-        #-- land-sea mask variable name and value of oceanic points
+        # land-sea mask variable name and value of oceanic points
         MASKNAME = 'lsm'
         OCEAN = 0
         GRAVITY = 9.80665
     elif (MODEL == 'ERA5'):
-        #-- mean file from calculate_mean_pressure.py
+        # mean file from calculate_mean_pressure.py
         input_mean_file = 'ERA5-Mean-SP-{0:4d}-{1:4d}.nc'
-        #-- input land-sea mask for ocean redistribution
+        # input land-sea mask for ocean redistribution
         input_mask_file = 'ERA5-Invariant-Parameters.nc'
-        #-- regular expression pattern for finding files
+        # regular expression pattern for finding files
         regex_pattern = r'ERA5\-Monthly\-SP\-({0})\.nc$'
         VARNAME = 'sp'
         LONNAME = 'longitude'
         LATNAME = 'latitude'
         TIMENAME = 'time'
         ELLIPSOID = 'WGS84'
-        #-- land-sea mask variable name and value of oceanic points
+        # land-sea mask variable name and value of oceanic points
         MASKNAME = 'lsm'
         OCEAN = 0
         GRAVITY = 9.80665
     elif (MODEL == 'MERRA-2'):
-        #-- mean file from calculate_mean_pressure.py
+        # mean file from calculate_mean_pressure.py
         input_mean_file = 'MERRA2.Mean_PS.{0:4d}-{1:4d}.nc'
-        #-- input land-sea mask for ocean redistribution
+        # input land-sea mask for ocean redistribution
         input_mask_file = 'MERRA2_101.const_2d_asm_Nx.00000000.nc4'
-        #-- regular expression pattern for finding files
+        # regular expression pattern for finding files
         regex_pattern = r'MERRA2_\d{{3}}.tavgM_2d_slv_Nx.({0})(\d{{2}}).(.*?).nc$'
         VARNAME = 'PS'
         LONNAME = 'lon'
         LATNAME = 'lat'
         TIMENAME = 'time'
         ELLIPSOID = 'WGS84'
-        #-- land-sea mask variable name and value of oceanic points
+        # land-sea mask variable name and value of oceanic points
         MASKNAME = 'FROCEAN'
         OCEAN = 1
         GRAVITY = 9.80665
     elif (MODEL == 'NCEP-DOE-2'):
-        #-- mean file from calculate_mean_pressure.py
+        # mean file from calculate_mean_pressure.py
         input_mean_file = 'pres.sfc.mean.{0:4d}-{1:4d}.nc'
-        #-- input land-sea mask for ocean redistribution
+        # input land-sea mask for ocean redistribution
         input_mask_file = 'land.nc'
-        #-- regular expression pattern for finding files
+        # regular expression pattern for finding files
         regex_pattern = r'pres.sfc.mon.mean.({0}).nc$'
         VARNAME = 'pres'
         LONNAME = 'lon'
         LATNAME = 'lat'
         TIMENAME = 'time'
         ELLIPSOID = 'WGS84'
-        #-- land-sea mask variable name and value of oceanic points
+        # land-sea mask variable name and value of oceanic points
         MASKNAME = 'land'
         OCEAN = 0
-        #-- NCEP-DOE-2 reanalysis geopotential heights are already in meters
+        # NCEP-DOE-2 reanalysis geopotential heights are already in meters
         GRAVITY = 1.0
     elif (MODEL == 'NCEP-CFSR'):
-        #-- mean file from calculate_mean_pressure.py
+        # mean file from calculate_mean_pressure.py
         input_mean_file = 'pgbh.mean.gdas.{0:4d}-{1:4d}.nc'
-        #-- input land-sea mask for ocean redistribution
+        # input land-sea mask for ocean redistribution
         input_mask_file = 'land.gdas.nc'
-        #-- regular expression pattern for finding files
+        # regular expression pattern for finding files
         regex_pattern = r'pgbh.gdas.({0}).nc$'
         VARNAME = 'PRES_L1_Avg'
         LONNAME = 'lon'
         LATNAME = 'lat'
         TIMENAME = 'time'
         ELLIPSOID = 'WGS84'
-        #-- land-sea mask variable name and value of oceanic points
+        # land-sea mask variable name and value of oceanic points
         MASKNAME = 'LAND_L1'
         OCEAN = 0
-        #-- NCEP-CFSR reanalysis geopotential heights are already in meters
+        # NCEP-CFSR reanalysis geopotential heights are already in meters
         GRAVITY = 1.0
     elif (MODEL == 'JRA-55'):
-        #-- mean file from calculate_mean_pressure.py
+        # mean file from calculate_mean_pressure.py
         input_mean_file = 'anl_surf.001_pres.mean.{0:4d}-{1:4d}.nc'
-        #-- input land-sea mask for ocean redistribution
+        # input land-sea mask for ocean redistribution
         input_mask_file = 'll125.081_land.2000.nc'
-        #-- regular expression pattern for finding files
+        # regular expression pattern for finding files
         regex_pattern = r'anl_surf125\.001_pres\.({0}).nc$'
         VARNAME = 'Pressure_surface'
         LONNAME = 'g0_lon_1'
         LATNAME = 'g0_lat_0'
         TIMENAME = 'time'
         ELLIPSOID = 'WGS84'
-        #-- land-sea mask variable name and value of oceanic points
+        # land-sea mask variable name and value of oceanic points
         MASKNAME = 'LAND_GDS0_SFC'
         OCEAN = 0
         GRAVITY = 9.80665
 
-    #-- upper bound of spherical harmonic orders (default = LMAX)
+    # upper bound of spherical harmonic orders (default = LMAX)
     MMAX = np.copy(LMAX) if not MMAX else MMAX
-    #-- output string for both LMAX == MMAX and LMAX != MMAX cases
-    order_str = 'M{0:d}'.format(MMAX) if (MMAX != LMAX) else ''
-    #-- if redistributing oceanic values to a mean value
+    # output string for both LMAX == MMAX and LMAX != MMAX cases
+    order_str = 'M{MMAX:d}' if (MMAX != LMAX) else ''
+    # if redistributing oceanic values to a mean value
     ocean_str = '_OCN' if REDISTRIBUTE else ''
-    #-- output suffix for data formats
+    # output suffix for data formats
     suffix = dict(ascii='txt',netCDF4='nc',HDF5='H5')
-    #-- output subdirectory
+    # output subdirectory
     args = (MODEL.upper(),LMAX,order_str,ocean_str)
     output_sub = '{0}_CLM_L{1:d}{2}{3}'.format(*args)
     if not os.access(os.path.join(ddir,output_sub),os.F_OK):
         os.makedirs(os.path.join(ddir,output_sub))
+    # attributes for output files
+    attributes = {}
+    attributes['reference'] = f'Output from {os.path.basename(sys.argv[0])}'
 
-    #-- read mean pressure field from calculate_mean_pressure.py
+    # read mean pressure field from calculate_mean_pressure.py
     mean_file = os.path.join(ddir,input_mean_file.format(RANGE[0],RANGE[1]))
     mean_pressure,lon,lat=ncdf_mean_pressure(mean_file,VARNAME,LONNAME,LATNAME)
     nlat,nlon = np.shape(mean_pressure)
-    #-- calculate colatitude
+    # calculate colatitude
     theta = (90.0 - lat)*np.pi/180.0
-    #-- calculate meshgrid from latitude and longitude
+    # calculate meshgrid from latitude and longitude
     gridlon,gridlat = np.meshgrid(lon,lat)
     gridphi = gridlon*np.pi/180.0
     gridtheta = (90.0 - gridlat)*np.pi/180.0
 
-    #-- read load love numbers and calculate Legendre polynomials
-    LOVE = load_love_numbers(LMAX,LOVE_NUMBERS=LOVE_NUMBERS,REFERENCE=REFERENCE)
-    PLM,dPLM = plm_holmes(LMAX,np.cos(theta))
+    # read load love numbers and calculate Legendre polynomials
+    LOVE = gravtk.load_love_numbers(LMAX, LOVE_NUMBERS=LOVE_NUMBERS,
+        REFERENCE=REFERENCE)
+    PLM, dPLM = gravtk.plm_holmes(LMAX, np.cos(theta))
 
-    #-- Earth Parameters
-    ellipsoid_params = ref_ellipsoid(ELLIPSOID)
-    #-- semimajor and semiminor axes of ellipsoid [m]
-    a_axis = ellipsoid_params['a']
-    b_axis = ellipsoid_params['b']
+    # get reference parameters for ellipsoid
+    ellipsoid_params = mdlhmc.constants(ellipsoid=ELLIPSOID)
+    # semimajor and semiminor axes of the ellipsoid [m]
+    a_axis = ellipsoid_params.a_axis
+    b_axis = ellipsoid_params.b_axis
 
-    #-- step size in radians
+    # step size in radians
     gridstep = np.zeros((2))
     gridstep[0] = np.abs(lon[1] - lon[0])
     gridstep[1] = np.abs(lat[1] - lat[0])
     dphi = np.pi*gridstep[0]/180.0
     dth = np.pi*gridstep[1]/180.0
-    #-- calculate grid areas globally
+    # calculate grid areas globally
     AREA = dphi*dth*np.sin(gridtheta)*np.sqrt((a_axis**2)*(b_axis**2) *
         ((np.sin(gridtheta)**2)*(np.cos(gridphi)**2) +
         (np.sin(gridtheta)**2)*(np.sin(gridphi)**2)) +
         (a_axis**4)*(np.cos(gridtheta)**2))
 
-    #-- get indices of land-sea mask if redistributing oceanic points
+    # get indices of land-sea mask if redistributing oceanic points
     if REDISTRIBUTE:
         ii,jj = ncdf_landmask(os.path.join(ddir,input_mask_file),MASKNAME,OCEAN)
-        #-- calculate total area of oceanic points
+        # calculate total area of oceanic points
         TOTAL_AREA = np.sum(AREA[ii,jj])
 
-    #-- read each reanalysis pressure field and convert to spherical harmonics
+    # read each reanalysis pressure field and convert to spherical harmonics
     regex_years = r'\d{4}' if (YEARS is None) else '|'.join(map(str,YEARS))
     rx = re.compile(regex_pattern.format(regex_years),re.VERBOSE)
     input_files = sorted([fi for fi in os.listdir(ddir) if rx.match(fi)])
-    #-- open output date and index files
-    output_date_file = '{0}_DATES.txt'.format(MODEL.upper())
-    fid1 = open(os.path.join(ddir,output_sub,output_date_file),'w')
-    output_index_file = '{0}_index.txt'.format(MODEL)
-    fid2 = open(os.path.join(ddir,output_sub,output_index_file),'w')
-    #-- date file header information
+    # open output date and index files
+    output_date_file = f'{MODEL.upper()}_DATES.txt'
+    fid1 = open(os.path.join(ddir,output_sub,output_date_file),
+        mode='w', encoding='utf8')
+    output_index_file = f'{MODEL}_index.txt'
+    fid2 = open(os.path.join(ddir,output_sub,output_index_file),
+        mode='w', encoding='utf8')
+    # date file header information
     print('{0:8} {1:10}'.format('Month','Date'), file=fid1)
-    #-- output file format for spherical harmonic data
+    # output file format for spherical harmonic data
     output_file_format = '{0}_CLM_L{1:d}{2}_{3:03d}.{4}'
 
-    #-- for each reanalysis file
+    # for each reanalysis file
     for fi in input_files:
-        #-- read input data
+        # read input data
         with netCDF4.Dataset(os.path.join(ddir,fi),'r') as fileID:
-            #-- check dimensions for expver slice
+            # check dimensions for expver slice
             if (fileID.variables[VARNAME].ndim == 4):
                 pressure = ncdf_expver(fileID, VARNAME)
             else:
                 pressure = fileID.variables[VARNAME][:].copy()
-            #-- convert time to Modified Julian Days
+            # convert time to Modified Julian Days
             delta_time=np.copy(fileID.variables[TIMENAME][:])
             date_string=fileID.variables[TIMENAME].units
-            epoch,to_secs=gravity_toolkit.time.parse_date_string(date_string)
-            MJD=gravity_toolkit.time.convert_delta_time(delta_time*to_secs,
+            epoch,to_secs = gravtk.time.parse_date_string(date_string)
+            MJD = gravtk.time.convert_delta_time(delta_time*to_secs,
                 epoch1=epoch, epoch2=(1858,11,17,0,0,0), scale=1.0/86400.0)
-        #-- iterate over Julian days
+        # iterate over Julian days
         for t,JD in enumerate(MJD+2400000.5):
-            #-- calculate pressure/gravity ratio for month
+            # calculate pressure/gravity ratio for month
             PG = (pressure[t,:,:] - mean_pressure[:,:])/GRAVITY
-            #-- if redistributing oceanic pressure values
+            # if redistributing oceanic pressure values
             if REDISTRIBUTE:
                 PG[ii,jj] = np.sum(PG[ii,jj]*AREA[ii,jj])/TOTAL_AREA
-            #-- calculate pressure harmonics from pressure/gravity ratio
-            Ylms = gen_stokes(PG, lon, lat, LMAX=LMAX, MMAX=MMAX, UNITS=3,
-                PLM=PLM, LOVE=LOVE)
-            #-- convert julian dates to calendar then to year-decimal
-            YY,MM,DD,hh,mm,ss = gravity_toolkit.time.convert_julian(JD,
+            # calculate pressure harmonics from pressure/gravity ratio
+            Ylms = gravtk.gen_stokes(PG, lon, lat, LMAX=LMAX, MMAX=MMAX,
+                UNITS=3, PLM=PLM, LOVE=LOVE)
+            # convert julian dates to calendar then to year-decimal
+            YY,MM,DD,hh,mm,ss = gravtk.time.convert_julian(JD,
                 FORMAT='tuple')
-            Ylms.time,=gravity_toolkit.time.convert_calendar_decimal(YY,
+            Ylms.time, = gravtk.time.convert_calendar_decimal(YY,
                 MM, day=DD, hour=hh, minute=mm, second=ss)
-            #-- calculate GRACE month from calendar dates
-            Ylms.month = gravity_toolkit.time.calendar_to_grace(YY, MM)
-            #-- output data to file
+            # calculate GRACE month from calendar dates
+            Ylms.month = gravtk.time.calendar_to_grace(YY, MM)
+            # output data to file
             args = (MODEL.upper(),LMAX,order_str,Ylms.month,suffix[DATAFORM])
             FILE = output_file_format.format(*args)
-            Ylms.to_file(os.path.join(ddir,output_sub,FILE),format=DATAFORM)
-            #-- set the permissions level of the output file to MODE
+            Ylms.to_file(os.path.join(ddir,output_sub,FILE),
+                format=DATAFORM, **attributes)
+            # set the permissions level of the output file to MODE
             os.chmod(os.path.join(ddir,output_sub,FILE), MODE)
 
-    #-- output file format for spherical harmonic data
+    # output file format for spherical harmonic data
     args = (MODEL.upper(),LMAX,order_str,suffix[DATAFORM])
     output_regex = re.compile(r'{0}_CLM_L{1:d}{2}_(\d+).{3}'.format(*args))
-    #-- find all output harmonic files (not just ones created in run)
+    # find all output harmonic files (not just ones created in run)
     output_files = [fi for fi in os.listdir(os.path.join(ddir,output_sub))
         if re.match(output_regex,fi)]
     for fi in sorted(output_files):
-        #-- extract GRACE month
+        # extract GRACE month
         grace_month, = np.array(re.findall(output_regex,fi),dtype=np.int64)
-        YY,MM = gravity_toolkit.time.grace_to_calendar(grace_month)
-        tdec, = gravity_toolkit.time.convert_calendar_decimal(YY, MM)
-        #-- full path to output file
+        YY,MM = gravtk.time.grace_to_calendar(grace_month)
+        tdec, = gravtk.time.convert_calendar_decimal(YY, MM)
+        # full path to output file
         full_output_file = os.path.join(ddir,output_sub,fi)
-        #-- print date, GRACE month and calendar month to date file
+        # print date, GRACE month and calendar month to date file
         fid1.write('{0:11.6f} {1:03d} {2:02.0f}\n'.format(tdec,grace_month,MM))
-        #-- print output file to index
+        # print output file to index
         print(full_output_file.replace(os.path.expanduser('~'),'~'), file=fid2)
-    #-- close the date and index files
+    # close the date and index files
     fid1.close()
     fid2.close()
-    #-- set the permissions level of the output date and index files to MODE
+    # set the permissions level of the output date and index files to MODE
     os.chmod(os.path.join(ddir,output_sub,output_date_file), MODE)
     os.chmod(os.path.join(ddir,output_sub,output_index_file), MODE)
 
-#-- PURPOSE: read reanalysis mean pressure from calculate_mean_pressure.py
+# PURPOSE: read reanalysis mean pressure from calculate_mean_pressure.py
 def ncdf_mean_pressure(FILENAME,VARNAME,LONNAME,LATNAME):
     with netCDF4.Dataset(FILENAME,'r') as fileID:
         mean_pressure = np.array(fileID.variables[VARNAME][:].squeeze())
@@ -360,32 +365,32 @@ def ncdf_mean_pressure(FILENAME,VARNAME,LONNAME,LATNAME):
         latitude = fileID.variables[LATNAME][:].squeeze()
     return (mean_pressure,longitude,latitude)
 
-#-- PURPOSE: extract pressure variable from a 4d netCDF4 dataset
-#-- ERA5 expver dimension (denotes mix of ERA5 and ERA5T)
+# PURPOSE: extract pressure variable from a 4d netCDF4 dataset
+# ERA5 expver dimension (denotes mix of ERA5 and ERA5T)
 def ncdf_expver(fileID, VARNAME):
     ntime,nexp,nlat,nlon = fileID.variables[VARNAME].shape
     fill_value = fileID.variables[VARNAME]._FillValue
-    #-- reduced surface pressure output
+    # reduced surface pressure output
     pressure = np.ma.zeros((ntime,nlat,nlon))
     pressure.fill_value = fill_value
     for t in range(ntime):
-        #-- iterate over expver slices to find valid outputs
+        # iterate over expver slices to find valid outputs
         for j in range(nexp):
-            #-- check if any are valid for expver
+            # check if any are valid for expver
             if np.any(fileID.variables[VARNAME][t,j,:,:]):
                 pressure[t,:,:] = fileID.variables[VARNAME][t,j,:,:]
-    #-- update mask variable
+    # update mask variable
     pressure.mask = (pressure.data == pressure.fill_value)
-    #-- return the reduced pressure variable
+    # return the reduced pressure variable
     return pressure
 
-#-- PURPOSE: read land sea mask to get indices of oceanic values
+# PURPOSE: read land sea mask to get indices of oceanic values
 def ncdf_landmask(FILENAME,MASKNAME,OCEAN):
     with netCDF4.Dataset(FILENAME,'r') as fileID:
         landsea = np.squeeze(fileID.variables[MASKNAME][:].copy())
     return np.nonzero(landsea == OCEAN)
 
-#-- PURPOSE: create argument parser
+# PURPOSE: create argument parser
 def arguments():
     parser = argparse.ArgumentParser(
         description="""Reads atmospheric surface pressure fields from
@@ -394,84 +399,84 @@ def arguments():
             """,
         fromfile_prefix_chars="@"
     )
-    parser.convert_arg_line_to_args = utilities.convert_arg_line_to_args
-    #-- command line parameters
+    parser.convert_arg_line_to_args = gravtk.utilities.convert_arg_line_to_args
+    # command line parameters
     choices = ['ERA-Interim','ERA5','MERRA-2','NCEP-DOE-2','NCEP-CFSR','JRA-55']
     parser.add_argument('model',
         type=str, nargs='+',
         default=['ERA5','MERRA-2'], choices=choices,
         help='Reanalysis Model')
-    #-- working data directory
+    # working data directory
     parser.add_argument('--directory','-D',
         type=lambda p: os.path.abspath(os.path.expanduser(p)),
         default=os.getcwd(),
         help='Working data directory')
-    #-- years to run
+    # years to run
     now = datetime.datetime.now()
     parser.add_argument('--year','-Y',
         type=int, nargs='+', default=range(2000,now.year+1),
         help='Years of model outputs to run')
-    #-- start and end years to run for mean
+    # start and end years to run for mean
     parser.add_argument('--mean',
         metavar=('START','END'), type=int, nargs=2,
         default=[2001,2002],
         help='Start and end year range for mean')
-    #-- uniformly redistribute pressure values over the ocean
+    # uniformly redistribute pressure values over the ocean
     parser.add_argument('--redistribute',
         default=False, action='store_true',
         help='Redistribute pressure values over the ocean')
-    #-- maximum spherical harmonic degree and order
+    # maximum spherical harmonic degree and order
     parser.add_argument('--lmax','-l',
         type=int, default=60,
         help='Maximum spherical harmonic degree')
     parser.add_argument('--mmax','-m',
         type=int, default=None,
         help='Maximum spherical harmonic order')
-    #-- different treatments of the load Love numbers
-    #-- 0: Han and Wahr (1995) values from PREM
-    #-- 1: Gegout (2005) values from PREM
-    #-- 2: Wang et al. (2012) values from PREM
+    # different treatments of the load Love numbers
+    # 0: Han and Wahr (1995) values from PREM
+    # 1: Gegout (2005) values from PREM
+    # 2: Wang et al. (2012) values from PREM
     parser.add_argument('--love','-n',
         type=int, default=0, choices=[0,1,2],
         help='Treatment of the Load Love numbers')
-    #-- option for setting reference frame for gravitational load love number
-    #-- reference frame options (CF, CM, CE)
+    # option for setting reference frame for gravitational load love number
+    # reference frame options (CF, CM, CE)
     parser.add_argument('--reference',
         type=str.upper, default='CF', choices=['CF','CM','CE'],
         help='Reference frame for load Love numbers')
-    #-- input and output data format (ascii, netCDF4, HDF5)
+    # input and output data format (ascii, netCDF4, HDF5)
     parser.add_argument('--format','-F',
         type=str, default='netCDF4', choices=['ascii','netCDF4','HDF5'],
         help='Input and output data format')
-    #-- print information about each input and output file
+    # print information about each input and output file
     parser.add_argument('--verbose','-V',
         action='count', default=0,
         help='Verbose output of processing run')
-    #-- permissions mode of the local directories and files (number in octal)
+    # permissions mode of the local directories and files (number in octal)
     parser.add_argument('--mode','-M',
         type=lambda x: int(x,base=8), default=0o775,
         help='Permission mode of directories and files')
-    #-- return the parser
+    # return the parser
     return parser
 
-#-- This is the main part of the program that calls the individual functions
+# This is the main part of the program that calls the individual functions
 def main():
-    #-- Read the system arguments listed after the program
+    # Read the system arguments listed after the program
     parser = arguments()
     args,_ = parser.parse_known_args()
 
-    #-- create logger
+    # create logger
     loglevels = [logging.CRITICAL,logging.INFO,logging.DEBUG]
     logging.basicConfig(level=loglevels[args.verbose])
 
-    #-- for each reanalysis model
+    # for each reanalysis model
     for MODEL in args.model:
-        #-- run program
+        # run program
         reanalysis_monthly_harmonics(args.directory, MODEL, args.year,
             RANGE=args.mean, REDISTRIBUTE=args.redistribute, LMAX=args.lmax,
             MMAX=args.mmax, LOVE_NUMBERS=args.love, REFERENCE=args.reference,
             DATAFORM=args.format, MODE=args.mode)
 
-#-- run main program
+# run main program
 if __name__ == '__main__':
     main()
